<?php

namespace Pinpoint\PackingSlips\Controller\Adminhtml\Generate;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;

/**
 * Class BulkPrint
 * @package Pinpoint\PackingSlips\Controller\Adminhtml\Generate
 */
class BulkPrint extends Action
{
    /**
     * @var \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip
     */
    protected $pickingSlip;

    /**
     * @var \iio\libmergepdf\Merger
     */
    protected $pdfMerger;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Collection
     */
    protected $orderCollection;

    /**
     * BulkPrint constructor.
     * @param Context $context
     * @param \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip $pickingSlip
     * @param \iio\libmergepdf\Merger $merger
     * @param \Magento\Sales\Model\ResourceModel\Order\Collection $orderCollection
     */
    public function __construct(
        Context $context,
        \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip $pickingSlip,
        \iio\libmergepdf\Merger $merger,
        \Magento\Sales\Model\ResourceModel\Order\Collection $orderCollection
    )
    {
        $this->pickingSlip = $pickingSlip;
        $this->pdfMerger = $merger;
        $this->orderCollection = $orderCollection;

        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('');
    }

    /**
     * @return bool|\Magento\Framework\App\ResponseInterface
     */
    public function execute()
    {
        error_reporting(0);
        $orderIds = $this->getRequest()->getParam('selected', []);

        if (!count($orderIds)) {
            $orderIds = array_values($this->orderCollection->getAllIds());
        }

        $pdfs = [];
        $machines = ['Priority Orders' => []];

        foreach ($orderIds as $orderId) {
            $pdfInfo = $this->pickingSlip->generateSlip($orderId, false, true);

            if (!$pdfInfo) {
                continue;
            }

            /** @var \Magento\Sales\Model\Order $order */
            $order = $pdfInfo[2];

            $nextDayDelivery = [
                'DPD NEXT WORKING DAY DELIVERY',
                'TRACKED ROYAL MAIL24',
                'Special Delivery - Order Before 2pm',
                'Saturday Royal Mail - Order Before 2pm',
                'Royal Mail - Special Delivery - Order Before 2pm'
            ];

            if ($this->strpos_arr($order->getShippingDescription(), $nextDayDelivery)) {
                $pdfInfo[1] = 'Priority Orders';
            }

            if (!isset($machines[$pdfInfo[1]])) {
                $machines[$pdfInfo[1]] = [];
            }

            $machines[$pdfInfo[1]][] = $pdfInfo[0];
        }

        if (count($machines)) {
            //put simple (no personalisation, next day etc) products last
            if (isset($machines['simple'])) {
                $simpleOrders = $machines['simple'];
                unset($machines['simple']);
                $machines['simple'] = $simpleOrders;
            }

            if (!count($machines['Priority Orders'])) {
                unset($machines['Priority Orders']);
            }

            foreach ($machines as $machine => $orders) {
                $pdfs[] = $this->pickingSlip->createSplitterPage($machine);

                foreach ($orders as $order) {
                    $pdfs[] = $order;
                }
            }

            $merger = $this->pdfMerger;

            foreach ($pdfs as $pdf) {
                $merger->addRaw($pdf->output());
            }

            $content = $merger->merge();

            header('Content-Type: application/pdf');
            header('Content-Length: ' . strlen($content));
            header('Content-disposition: attachment; filename="bulkprint-' . date('d-m-Y') . '.pdf"');
            header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');

            echo $content;
            return true;
        }

        return $this->_redirect('sales/order/index');
    }

    /**
    * runs a strpos() on an array of needles
    * @return boolean
    */
    protected function strpos_arr($haystack, $needle)
    {
        if (!is_array($needle)) $needle = array($needle);

        foreach ($needle as $what) {
            if (($pos = strpos($haystack, $what)) !== false) return true;
        }

        return false;
    }
}
